package it.unimi.dsi.webgraph;


/*		 
 * Copyright (C) 2003, 2004, 2005 Paolo Boldi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

import it.unimi.dsi.fastutil.ints.AbstractIntIterator;
import it.unimi.dsi.fastutil.ints.IntIterator;
import it.unimi.dsi.fastutil.ints.IntIterators;

/** This interface extends {@link IntIterator} and is used to scan a graph, that is, to read its nodes and their successor lists
 *  sequentially. The {@link #nextInt()} method returns the node that will be scanned. After a call to this method,  calling
 *  {@link #successors()} will return the list of successors.
 */

public abstract class NodeIterator extends AbstractIntIterator {
	
	/** Returns the outdegree of the current node.
	 *
	 *  @return the outdegree of the current node.
	 */
	public abstract int outdegree();

	/** Returns an iterator over the successors of the current node. A subclass may decide not to override this method,
	 *  or to override the {@link #successorArray()} method instead, but at least one of them <em>must</em> be overridden.
	 *
	 *  @return an iterator over the successors of the current node.
	 */
	public IntIterator successors() {
		return IntIterators.wrap( successorArray(), 0, outdegree() );
	}

	/** Returns a reference to an array containing the successors of a given node.
	 * 
	 * <P>The returned array may contain more entries than the outdegree of <code>x</code>.
	 * However, only those with indices from 0 (inclusive) to the outdegree of <code>x</code> (exclusive)
	 * contain valid data.
	 * 
	 * <P>This implementation just unwrap the iterator returned by {@link #successors()}.
	 * 
	 * @return an array whose first elements are the successors of the node; the array must not
	 * be modified by the caller.
	 */
	public int[] successorArray() {
		return IntIterators.unwrap( successors() );
	}
}

// Local Variables:
// mode: jde
// tab-width: 4
// End:
